@@ -107,10 +107,14 @@ typedef struct dt_probe		dt_probe_t;
  * with a NULL function name.  These probes are used to validate tracepoints
  * that are found in the actual code.
  *
- * During tracepoint validation, probes with actual fuction names will be added
- * to the pmap hashtable.  These probes will have tracepoint data associated
- * with them, and are the probes that will be emitted as parsed data for the
- * provider.  Any probes that do not have tracepoints will be ignored.
+ * During tracepoint validation, tracepoint probes (with actual fuction names)
+ * will be moved from the prbmap hashtable to the pvp->pmap hashtable.  These
+ * probes have tracepoint data associated with them, and are the probes that
+ * will be emitted as parsed data for the provider.  Any probes that do not
+ * have tracepoints will be ignored.
+ *
+ * The pvp->pmap hashtable therefore will require a specific cleanup function
+ * to ensure that the probe data is freed.
  *
  * The dt_provider_t.probec tracks the number of probes with tracepoints.
  */
@@ -161,7 +165,29 @@ static int prv_cmp(const dt_provider_t *p, const dt_provider_t *q) {
 }
 
 DEFINE_HE_STD_LINK_FUNCS(prv, dt_provider_t, he)
-DEFINE_HTAB_STD_OPS(prv)
+
+/*
+ * Hashtable element cleanup function for providers.  It ensures that the probe
+ * hashtable (pvp->pmap) is destroyed.
+ */
+static void *
+prv_del_prov(dt_provider_t *head, dt_provider_t *pvp)
+{
+	head = prv_del(head, pvp);
+
+	dt_htab_destroy(pvp->pmap);
+	free(pvp);
+
+	return head;
+}
+
+static dt_htab_ops_t prv_htab_ops = {
+        .hval = (htab_hval_fn)prv_hval,
+        .cmp = (htab_cmp_fn)prv_cmp,
+        .add = (htab_add_fn)prv_add,
+        .del = (htab_del_fn)prv_del_prov,
+        .next = (htab_next_fn)prv_next
+};
 
 static uint32_t prb_hval(const dt_probe_t *prp) {
 	uint32_t	hval;
@@ -205,6 +231,47 @@ static int prb_cmp(const dt_probe_t *p, const dt_probe_t *q) {
 DEFINE_HE_STD_LINK_FUNCS(prb, dt_probe_t, he)
 DEFINE_HTAB_STD_OPS(prb)
 
+/*
+ * Probe hashtable element cleanup function to ensure that probe data is freed
+ * when probes are removed from the pvp->pmap hashtable.  Note that probes in
+ * the prbmap hashtable do *not* get freed upon removal because they get moved
+ * to a pvp->pmap hashtable, and they will get freed when removed from there.
+ */
+static void *
+prb_del_probe(dt_probe_t *head, dt_probe_t *prp)
+{
+	head = prb_del(head, prp);
+
+	/*
+	 * If this is not a function-specific probe (from a prov note), free
+	 * the translated arg data and the probe itself.
+	 * If this is a function-specific probe (from a usdt note), walk the
+	 * list of tracepoint probe, freeing each probe in the list.
+	 */
+	if (prp->fun == NULL) {
+		free(prp->xargs);
+		free(prp->xmap);
+		free(prp);
+	} else {
+		dt_probe_t	*nxt;
+
+		do {
+			nxt = prp->next;
+			free(prp);
+		} while ((prp = nxt) != NULL);
+	}
+
+	return head;
+}
+
+static dt_htab_ops_t pmap_htab_ops = {
+        .hval = (htab_hval_fn)prb_hval,
+        .cmp = (htab_cmp_fn)prb_cmp,
+        .add = (htab_add_fn)prb_add,
+        .del = (htab_del_fn)prb_del_probe,
+        .next = (htab_next_fn)prb_next
+};
+
 /*
  * Return the cummulative string length of 'cnt' consecutive 0-terminated
  * strings.  If skip > 0, it indicates how many extra bytes are to be skipped
@@ -220,7 +287,6 @@ strarray_size(uint8_t cnt, const char *str, const char *end, size_t skip)
 		if (p >= end)
 			return -1;
 
-dt_dbg_usdt("%s: [%s] [%hhd]\n", __func__, p, *(p + strlen(p) + 1));
 		p += strlen(p) + 1 + skip;
 	}
 
@@ -252,7 +318,7 @@ parse_prov_note(int out, dof_helper_t *dhp, usdt_data_t *data,
 		memset(pvp, 0, sizeof(dt_provider_t));
 		pvp->name = prvt.name;
 		dt_htab_insert(prvmap, pvp);
-		pvp->pmap = dt_htab_create(&prb_htab_ops);
+		pvp->pmap = dt_htab_create(&pmap_htab_ops);
 	} else {
 		usdt_error(out, EEXIST, "Duplicate provider: %s", prvt.name);
 		return -1;
@@ -405,6 +471,23 @@ parse_usdt_note(int out, dof_helper_t *dhp, usdt_data_t *data,
 	}
 	prbt.off = off;
 
+	/*
+	 * If the probe name has encoded hyphens, perform in-place changing
+	 * from "__" into "-".
+	 */
+	if (strstr(prbt.prb, "__") != NULL) {
+		char		*q;
+		const char	*s = prbt.prb, *e = p;
+
+		for (q = (char *)s; s < e; s++, q++) {
+			if (s[0] == '_' && s[1] == '_') {
+				*q = '-';
+				s++;
+			} else if (s > q)
+				*q = *s;
+		}
+	}
+
 	if ((prp = dt_htab_lookup(prbmap, &prbt)) == NULL) {
 		if ((prp = malloc(sizeof(dt_probe_t))) == NULL) {
 			usdt_error(out, ENOMEM, "Failed to allocate probe");
@@ -708,37 +791,32 @@ usdt_parse_notes(int out, dof_helper_t *dhp, usdt_data_t *data)
 		}
 
 		/*
-		 * The tracepoint is valid.  Add it to the provider.
+		 * The tracepoint is valid.  Remove it from the prbmap htab and
+		 * add it to the provider.
 		 * If there was a matching function-specific probe, add the
 		 * tracepoint probe to it.
 		 * If there was no matching function-specific probe, add the
 		 * tracepoint probe to the provider.
 		 * In either cases, argument data is copied.
 		 */
+		dt_htab_delete(prbmap, ptp);
 		if (prp->fun != NULL) {
 			ptp->next = prp->next;
-			ptp->nargc = prp->nargc;
-			ptp->nargs = prp->nargs;
-			ptp->nargsz = prp->nargsz;
-			ptp->xargc = prp->xargc;
-			ptp->xargs = prp->xargs;
-			ptp->xargsz = prp->xargsz;
-			ptp->xmap = prp->xmap;
 			prp->next = ptp;
 			prp->ntp++;
 		} else {
-			dt_htab_delete(prbmap, ptp);
 			dt_htab_insert(pvp->pmap, ptp);
 			ptp->ntp = 1;
-			ptp->nargc = prp->nargc;
-			ptp->nargs = prp->nargs;
-			ptp->nargsz = prp->nargsz;
-			ptp->xargc = prp->xargc;
-			ptp->xargs = prp->xargs;
-			ptp->xargsz = prp->xargsz;
-			ptp->xmap = prp->xmap;
 			pvp->probec++;
 		}
+
+		ptp->nargc = prp->nargc;
+		ptp->nargs = prp->nargs;
+		ptp->nargsz = prp->nargsz;
+		ptp->xargc = prp->xargc;
+		ptp->xargs = prp->xargs;
+		ptp->xargsz = prp->xargsz;
+		ptp->xmap = prp->xmap;
 	}
 
 	/* Emit any provider that has tracepoints. */
@@ -751,6 +829,13 @@ usdt_parse_notes(int out, dof_helper_t *dhp, usdt_data_t *data)
 	goto out;
 
 err:
+	/*
+	 * In case of an error, we don't know whether we can allocate iterators
+	 * so we have few options to provide cleanup.  Since cleanup is all we
+	 * do now, we plug the probe cleanup function into the ops for prbmap,
+	 * and let the hashtable destroy function take care of all cleanup.
+	 */
+	prb_htab_ops.del = (htab_del_fn)prb_del_probe;
 	rc = -1;
 
 out:
