@@ -523,7 +523,7 @@ dt_bpf_init_features(dtrace_hdl_t *dtp)
 {
 	uint32_t	btf_id;
 
-	btf_id = dt_btf_lookup_name_kind(dtp, dtp->dt_shared_btf,
+	btf_id = dt_btf_lookup_name_kind(dtp, dtp->dt_exec,
 					 "bpf_get_btf_vmlinux", BTF_KIND_FUNC);
 	if (btf_id >= 0 &&
 	    have_attach_type(BPF_PROG_TYPE_TRACING, BPF_TRACE_FENTRY, btf_id))
@@ -689,6 +689,10 @@ gmap_create_aggs(dtrace_hdl_t *dtp)
 	if (dtp->dt_aggmap_fd == -1)
 		return -1;
 
+	dtp->dt_aggmap_ids = dt_calloc(dtp, dtp->dt_conf.num_online_cpus, sizeof(int));
+	if (dtp->dt_aggmap_ids == NULL)
+		return dt_set_errno(dtp, EDT_NOMEM);
+
 	for (i = 0; i < dtp->dt_conf.num_online_cpus; i++) {
 		int	cpu = dtp->dt_conf.cpus[i].cpu_id;
 		char	name[16];
@@ -702,6 +706,8 @@ gmap_create_aggs(dtrace_hdl_t *dtp)
 			return map_create_error(dtp, name, errno);
 
 		dt_bpf_map_update(dtp->dt_aggmap_fd, &cpu, &fd);
+		if (dt_bpf_map_lookup(dtp->dt_aggmap_fd, &cpu, &dtp->dt_aggmap_ids[i]) < 0)
+			return -1;
 	}
 
 	/* Create the agg generation value array. */
@@ -904,8 +910,8 @@ gmap_create_strtab(dtrace_hdl_t *dtp)
 	while (buf < end) {
 		uint_t	len = strlen((char *)buf);
 
-		if (len > strsize)
-			buf[strsize] = '\0';
+		if (len >= strsize)
+			buf[strsize - 1] = '\0';
 
 		buf += len + 1;
 	}
@@ -1333,6 +1339,16 @@ dt_bpf_load_progs(dtrace_hdl_t *dtp, uint_t cflags)
 	 */
 	dtrace_getopt(dtp, "destructive", &dest_ok);
 
+	/*
+	 * If we have any destructive actions at all and -w is not set,
+	 * error out.  Solaris would reject this as a runtime error.  So,
+	 * although we could have detected this problem at compilation,
+	 * we mimic Solaris and wait until now to report.
+	 */
+	if (dtp->dt_destructive && dest_ok == DTRACEOPT_UNSET)
+		return dt_set_errno(dtp, EDT_DESTRUCTIVE);
+
+	dp = NULL;
 	for (prp = dt_list_next(&dtp->dt_enablings); prp != NULL;
 	     prp = dt_list_next(prp)) {
 		int		fd;
@@ -1347,28 +1363,44 @@ dt_bpf_load_progs(dtrace_hdl_t *dtp, uint_t cflags)
 			continue;
 
 		if (dt_link(dtp, prp, dp, NULL) == -1)
-			return -1;
+			goto fail;
 
 		DT_DISASM_PROG_LINKED(dtp, cflags, dp, stderr, NULL, prp->desc);
 
+		/*
+		 * This check should never fail since, if any action is
+		 * destructive and -w is not set, we should already have
+		 * failed.
+		 */
 		if (dp->dtdo_flags & DIFOFLG_DESTRUCTIVE &&
-		    dest_ok == DTRACEOPT_UNSET)
-			return dt_set_errno(dtp, EDT_DESTRUCTIVE);
+		    dest_ok == DTRACEOPT_UNSET) {
+			dt_set_errno(dtp, EDT_DESTRUCTIVE);
+			goto fail;
+		}
 
 		fd = dt_bpf_load_prog(dtp, prp, dp, cflags);
 		if (fd == -1)
-			return -1;
+			goto fail;
 
 		if (prp->prov->impl->attach)
 			rc = prp->prov->impl->attach(dtp, prp, fd);
 
-		if (rc < 0) {
+		if (rc < 0 && !(prp->flags & DT_PROBE_FLAG_OPTIONAL)) {
 			close(fd);
-			return dt_attach_error(dtp, rc,
-					       prp->desc->prv, prp->desc->mod,
-					       prp->desc->fun, prp->desc->prb);
+			dt_attach_error(dtp, rc,
+					prp->desc->prv, prp->desc->mod,
+					prp->desc->fun, prp->desc->prb);
+			goto fail;
 		}
+
+		dt_difo_free(dtp, prp->difo);
+		prp->difo = NULL;
 	}
 
 	return 0;
+
+fail:
+	dt_difo_free(dtp, prp->difo);
+	prp->difo = NULL;
+	return -1;
 }
