@@ -250,11 +250,11 @@ get_asm_reg(dt_provider_t *pvp, const char *name)
 		     i++, rp++) {
 #if defined(__amd64)
 			rp->off = dt_cg_ctf_offsetof("struct pt_regs",
-						      rp->mname, NULL, 0) +
+						rp->mname, NULL, NULL, 0) +
 				  rp->moff;
 #elif defined(__aarch64__)
 			rp->off = dt_cg_ctf_offsetof("struct user_pt_regs",
-						      rp->mname, NULL, 0) +
+						rp->mname, NULL, NULL, 0) +
 				  rp->moff;
 #endif
 
@@ -283,6 +283,7 @@ typedef struct dt_uprobe {
 	char		*fn;		/* object full file name */
 	char		*func;		/* function */
 	uint64_t	off;
+	uint64_t	refcntr_off;	/* optional reference counter offset */
 	int		flags;
 	tp_probe_t	*tp;
 	int		argc;		/* number of args */
@@ -313,12 +314,15 @@ static const dtrace_pattr_t	pattr = {
 
 dt_provimpl_t	dt_pid;
 dt_provimpl_t	dt_usdt;
+dt_provimpl_t	dt_stapsdt;
 
 static int populate(dtrace_hdl_t *dtp)
 {
 	if (dt_provider_create(dtp, dt_uprobe.name, &dt_uprobe, &pattr,
 			       NULL) == NULL ||
 	    dt_provider_create(dtp, dt_pid.name, &dt_pid, &pattr,
+			       NULL) == NULL ||
+	    dt_provider_create(dtp, dt_stapsdt.name, &dt_stapsdt, &pattr,
 			       NULL) == NULL)
 		return -1;			/* errno already set */
 
@@ -477,8 +481,8 @@ clean_usdt_probes(dtrace_hdl_t *dtp)
 
 		prp_next = dt_list_next(prp);
 
-		/* Make sure it is an overlying USDT probe. */
-		if (prp->prov->impl != &dt_usdt)
+		/* Make sure it is an overlying USDT, stapsdt probe. */
+		if (prp->prov->impl != &dt_usdt && prp->prov->impl != &dt_stapsdt)
 			continue;
 
 		/* FIXME passing in NULL pcb and dpr wreaks havoc on error reporting? */
@@ -504,7 +508,8 @@ clean_usdt_probes(dtrace_hdl_t *dtp)
 
 /*
  * Judge whether clause "n" could ever be called as a USDT probe
- * for this underlying probe.
+ * for this underlying probe.  We can pass uprp==NULL to see if
+ * the clause can be excluded for every probe.
  */
 static int
 ignore_clause(dtrace_hdl_t *dtp, int n, const dt_probe_t *uprp)
@@ -512,6 +517,9 @@ ignore_clause(dtrace_hdl_t *dtp, int n, const dt_probe_t *uprp)
 	dtrace_stmtdesc_t	*stp = dtp->dt_stmts[n];
 	dtrace_probedesc_t	*pdp = &stp->dtsd_ecbdesc->dted_probe;
 
+	if (stp == NULL)
+		return 1;
+
 	/*
 	 * Some clauses could never be called for a USDT probe,
 	 * regardless of the underlying probe uprp.  Cache this
@@ -525,7 +533,7 @@ ignore_clause(dtrace_hdl_t *dtp, int n, const dt_probe_t *uprp)
 		 * neither '*' nor a digit, it cannot be a USDT probe.
 		 */
 		if (len > 1) {
-			char	lastchar = pdp->prv[len - 1];
+			char	lastchar = (pdp->prv[0] != '\0' ? pdp->prv[len - 1] : '*');
 
 			if (lastchar != '*' && !isdigit(lastchar)) {
 				dt_stmt_clsflag_set(stp, DT_CLSFLAG_USDT_EXCLUDE);
@@ -555,6 +563,8 @@ ignore_clause(dtrace_hdl_t *dtp, int n, const dt_probe_t *uprp)
 	}
 	if (dt_stmt_clsflag_test(stp, DT_CLSFLAG_USDT_EXCLUDE) == 1)
 		return 1;
+	if (uprp == NULL)
+		return 0;
 
 	/*
 	 * If we cannot ignore this statement, try to use uprp.
@@ -609,34 +619,43 @@ static int add_probe_uprobe(dtrace_hdl_t *dtp, dt_probe_t *prp)
 	/* Make program. */
 	dp = dt_construct(dtp, prp, cflags, NULL);
 	if (dp == NULL)
-		return 0;        // FIXME in dt_bpf_make_progs() this is a fatal error; should we do the same here?
+		return 0;
 	prp->difo = dp;
 
 	/* Load program. */
 	if (dt_link(dtp, prp, dp, NULL) == -1)
-		return 0;        // FIXME in dt_bpf_load_progs() this is a fatal error; should we do the same here?
+		goto fail;
 
 	dtrace_getopt(dtp, "destructive", &dest_ok);
 	if (dp->dtdo_flags & DIFOFLG_DESTRUCTIVE &&
-	    dest_ok == DTRACEOPT_UNSET)
-		return dt_set_errno(dtp, EDT_DESTRUCTIVE);
+	    dest_ok == DTRACEOPT_UNSET) {
+		dt_set_errno(dtp, EDT_DESTRUCTIVE);
+		goto fail;
+	}
 
 	fd = dt_bpf_load_prog(dtp, prp, dp, cflags);
 	if (fd == -1)
-		return 0;        // FIXME in dt_bpf_load_progs() this is a fatal error; should we do the same here?
+		goto fail;
 
 	if (prp->prov->impl->attach)
 		rc = prp->prov->impl->attach(dtp, prp, fd);
 
 	if (rc < 0) {
 		close(fd);
-		return dt_attach_error(dtp, rc, prp->desc->prv, prp->desc->mod,
-						prp->desc->fun, prp->desc->prb);
+		dt_attach_error(dtp, rc, prp->desc->prv, prp->desc->mod,
+					 prp->desc->fun, prp->desc->prb);
+		goto fail;
 	}
 
 	return 0;
+
+fail:
+	dt_difo_free(dtp, prp->difo);
+	prp->difo = NULL;
+	return 0;	// FIXME in dt_bpf_make_progs() this is a fatal error; should we do the same here?
 }
 
+/* shared between usdt, stapsdt probes */
 static int add_probe_usdt(dtrace_hdl_t *dtp, dt_probe_t *prp)
 {
 	char				probnam[DTRACE_FULLNAMELEN], *p;
@@ -751,13 +770,9 @@ static int discover(dtrace_hdl_t *dtp)
 	 */
 	memset(&pcb, 0, sizeof(dt_pcb_t));
 	for (i = 0; i < dtp->dt_stmt_nextid; i++) {
-		dtrace_stmtdesc_t *stp;
-
-		stp = dtp->dt_stmts[i];
-		if (stp == NULL)
+		if (ignore_clause(dtp, i, NULL))
 			continue;
-		if (dt_stmt_clsflag_test(stp, DT_CLSFLAG_USDT_EXCLUDE) != 1)
-			dt_pid_create_usdt_probes(&stp->dtsd_ecbdesc->dted_probe, dtp, &pcb);
+		dt_pid_create_usdt_probes(&dtp->dt_stmts[i]->dtsd_ecbdesc->dted_probe, dtp, &pcb);
 	}
 
 	return 0;
@@ -780,6 +795,10 @@ static int populate_args(dtrace_hdl_t *dtp, const pid_probespec_t *psp,
 	char	*nptr = NULL, *xptr = NULL;
 	size_t	i;
 
+	/* Nothing to do if we already populated the arguments. */
+	if (upp->argc >= 0)
+		return 0;
+
 	upp->argc = psp->pps_xargc;
 
 	/* Copy argument value source string data (if any). */
@@ -860,6 +879,7 @@ static dt_probe_t *create_underlying(dtrace_hdl_t *dtp,
 				     const pid_probespec_t *psp)
 {
 	char			mod[DTRACE_MODNAMELEN];
+	char			fun[DTRACE_FUNCNAMELEN];
 	char			prb[DTRACE_NAMELEN];
 	dtrace_probedesc_t	pd;
 	dt_probe_t		*uprp;
@@ -877,12 +897,14 @@ static dt_probe_t *create_underlying(dtrace_hdl_t *dtp,
 	 *
 	 * The probe description for return probes is:
 	 *
-	 *	uprobe:<dev>_<inode>::return
+	 *	uprobe:<dev>_<inode>:<func>:return
 	 */
 	snprintf(mod, sizeof(mod), "%lx_%lx", psp->pps_dev, psp->pps_inum);
 
+	fun[0] = '\0';
 	switch (psp->pps_type) {
 	case DTPPT_RETURN:
+		strcpy(fun, psp->pps_fun);
 		strcpy(prb, "return");
 		break;
 	case DTPPT_IS_ENABLED:
@@ -890,6 +912,7 @@ static dt_probe_t *create_underlying(dtrace_hdl_t *dtp,
 	case DTPPT_OFFSETS:
 	case DTPPT_ABSOFFSETS:
 	case DTPPT_USDT:
+	case DTPPT_STAPSDT:
 		snprintf(prb, sizeof(prb), "%lx", psp->pps_off);
 		break;
 	default:
@@ -900,11 +923,11 @@ static dt_probe_t *create_underlying(dtrace_hdl_t *dtp,
 	pd.id = DTRACE_IDNONE;
 	pd.prv = prvname;
 	pd.mod = mod;
-	pd.fun = "";
+	pd.fun = fun;
 	pd.prb = prb;
 
 	dt_dprintf("Providing underlying probe %s:%s:%s:%s @ %lx\n", psp->pps_prv,
-		   psp->pps_mod, psp->pps_fn, psp->pps_prb, psp->pps_off);
+		   psp->pps_mod, psp->pps_fun, psp->pps_prb, psp->pps_off);
 	uprp = dt_probe_lookup(dtp, &pd);
 	if (uprp == NULL) {
 		dt_provider_t	*pvp;
@@ -922,8 +945,10 @@ static dt_probe_t *create_underlying(dtrace_hdl_t *dtp,
 		upp->dev = psp->pps_dev;
 		upp->inum = psp->pps_inum;
 		upp->off = psp->pps_off;
+		upp->refcntr_off = psp->pps_refcntr_off;
 		upp->fn = strdup(psp->pps_fn);
 		upp->func = NULL;
+		upp->argc = -1;			/* no argument data yet */
 		upp->tp = dt_tp_alloc(dtp);
 		if (upp->tp == NULL)
 			goto fail;
@@ -932,9 +957,16 @@ static dt_probe_t *create_underlying(dtrace_hdl_t *dtp,
 				       upp);
 		if (uprp == NULL)
 			goto fail;
-	} else
+
+		if (psp->pps_flags & DT_PID_PSP_FLAG_OPTIONAL)
+			uprp->flags |= DT_PROBE_FLAG_OPTIONAL;
+	} else {
 		upp = uprp->prv_data;
 
+		if (!(psp->pps_flags & DT_PID_PSP_FLAG_OPTIONAL))
+			uprp->flags &= ~DT_PROBE_FLAG_OPTIONAL;
+	}
+
 	/*
 	 * Only one USDT probe can correspond to each underlying probe.
 	 */
@@ -951,8 +983,6 @@ static dt_probe_t *create_underlying(dtrace_hdl_t *dtp,
 	if (psp->pps_type != DTPPT_RETURN) {
 		if (upp->func == NULL)
 			upp->func = strdup(psp->pps_fun);
-		else
-			assert(strcmp(upp->func, psp->pps_fun) == 0);
 	}
 
 	switch (psp->pps_type) {
@@ -963,6 +993,7 @@ static dt_probe_t *create_underlying(dtrace_hdl_t *dtp,
 		upp->flags |= PP_IS_ENABLED;
 		break;
 	case DTPPT_USDT:
+	case DTPPT_STAPSDT:
 		upp->flags |= PP_IS_USDT;
 		break;
 	default: ;
@@ -971,8 +1002,10 @@ static dt_probe_t *create_underlying(dtrace_hdl_t *dtp,
 		 */
 	}
 
-	if (populate_args(dtp, psp, upp) < 0)
-		goto fail;
+	if (upp->flags & (PP_IS_ENABLED | PP_IS_USDT)) {
+		if (populate_args(dtp, psp, upp) < 0)
+			goto fail;
+	}
 
 	return uprp;
 
@@ -1023,6 +1056,12 @@ static int provide_probe(dtrace_hdl_t *dtp, const pid_probespec_t *psp,
 	/* Look up the overlying probe. */
 	prp = dt_probe_lookup(dtp, &pd);
 	if (prp != NULL) {
+		/*
+		 * If not optional, pass that info on.
+		 */
+		if (!(psp->pps_flags & DT_PID_PSP_FLAG_OPTIONAL))
+			prp->flags &= ~DT_PROBE_FLAG_OPTIONAL;
+
 		/*
 		 * Probe already exists.  If it's already in the underlying
 		 * probe's probe list, there is nothing left to do.
@@ -1053,10 +1092,17 @@ static int provide_probe(dtrace_hdl_t *dtp, const pid_probespec_t *psp,
 	 */
 
 	pup->probe = uprp;
-	if (prp == NULL)
+	if (prp == NULL) {
 		prp = dt_probe_insert(dtp, pvp, pd.prv, pd.mod, pd.fun, pd.prb,
 				      pup);
-	else
+
+		/*
+		 * If not optional, pass that info on.
+		 */
+		if (psp->pps_flags & DT_PID_PSP_FLAG_OPTIONAL)
+			prp->flags |= DT_PROBE_FLAG_OPTIONAL;
+
+	} else
 		dt_list_append((dt_list_t *)prp->prv_data, pup);
 
 	if (prp == NULL) {
@@ -1108,11 +1154,24 @@ static int provide_usdt_probe(dtrace_hdl_t *dtp, const pid_probespec_t *psp)
 	return provide_probe(dtp, psp, psp->pps_prb, &dt_usdt, PP_IS_FUNCALL);
 }
 
+static int provide_stapsdt_probe(dtrace_hdl_t *dtp, const pid_probespec_t *psp)
+{
+	if (psp->pps_type != DTPPT_STAPSDT &&
+	    psp->pps_type != DTPPT_IS_ENABLED) {
+		dt_dprintf("pid: unknown stapsdt probe type %i\n", psp->pps_type);
+		return -1;
+	}
+
+	return provide_probe(dtp, psp, psp->pps_prb, &dt_stapsdt, PP_IS_FUNCALL);
+}
+
+
 static void enable(dtrace_hdl_t *dtp, dt_probe_t *prp, int is_usdt)
 {
 	const list_probe_t	*pup;
 
-	assert(prp->prov->impl == &dt_pid || prp->prov->impl == &dt_usdt);
+	assert(prp->prov->impl == &dt_pid || prp->prov->impl == &dt_usdt ||
+	       prp->prov->impl == &dt_stapsdt);
 
 	/*
 	 * We need to enable the underlying probes (if not enabled yet).
@@ -1144,6 +1203,11 @@ static void enable_usdt(dtrace_hdl_t *dtp, dt_probe_t *prp)
 	enable(dtp, prp, 1);
 }
 
+static void enable_stapsdt(dtrace_hdl_t *dtp, dt_probe_t *prp)
+{
+	enable(dtp, prp, 1);
+}
+
 /*
  * Generate code that populates the probe arguments.
  */
@@ -1651,17 +1715,25 @@ static char *uprobe_name(dev_t dev, ino_t ino, uint64_t addr, int flags)
  * Create a uprobe for a given dev/ino, mapping filename, and address: the
  * uprobe may be a uretprobe.  Return the probe's name as
  * a new dynamically-allocated string, or NULL on error.
+ *
+ * An optional refcntr_off - used by stapsdt probes to identify semaphore
+ * address - can also be supplied.
  */
 static char *uprobe_create(dev_t dev, ino_t ino, const char *mapping_fn,
-			   uint64_t addr, int flags)
+			   uint64_t addr, uint64_t refcntr_off, int flags)
 {
 	int	fd = -1;
 	int	rc = -1;
 	char	*name;
 	char	*spec;
 
-	if (asprintf(&spec, "%s:0x%lx", mapping_fn, addr) < 0)
-		return NULL;
+	if (refcntr_off) {
+		if (asprintf(&spec, "%s:0x%lx(0x%lx)", mapping_fn, addr, refcntr_off) < 0)
+			return NULL;
+	} else {
+		if (asprintf(&spec, "%s:0x%lx", mapping_fn, addr) < 0)
+			return NULL;
+	}
 
 	name = uprobe_name(dev, ino, addr, flags);
 	if (!name)
@@ -1675,6 +1747,7 @@ static char *uprobe_create(dev_t dev, ino_t ino, const char *mapping_fn,
 	rc = dprintf(fd, "%c:%s %s\n", flags & PP_IS_RETURN ? 'r' : 'p', name, spec);
 
 out:
+	free(spec);
 	if (fd != -1)
 		close(fd);
 	if (rc < 0) {
@@ -1700,7 +1773,7 @@ static int attach(dtrace_hdl_t *dtp, const dt_probe_t *uprp, int bpf_fd)
 	assert(upp->fn != NULL);
 
 	prb = uprobe_create(upp->dev, upp->inum, upp->fn, upp->off,
-			    upp->flags);
+			    upp->refcntr_off, upp->flags);
 
 	/*
 	 * If the uprobe creation failed, it is possible it already
@@ -1781,6 +1854,106 @@ oom:
 	return dt_set_errno(dtp, EDT_NOMEM);
 }
 
+/*
+ * Return a representative datatype for an stapsdt argument.
+ * If 'ssize' is negative, the datatype is signed.  The absolute value gives
+ * the type size in bytes.
+ */
+static char *staptype(int ssize)
+{
+	int	rc, sign = 0;
+	char	*s;
+
+	if (ssize < 0) {
+		sign = 1;
+		ssize = -ssize;
+	}
+
+	switch (ssize) {
+	case 1:
+	case 2:
+		break;
+	case 3: case 4:
+		ssize = 4;
+		break;
+	default:
+		ssize = 8;
+		break;
+	}
+
+	rc = asprintf(&s, sign ? "int%d_t" : "uint%d_t", ssize * 8);
+
+	return rc == -1 ? NULL : s;
+}
+
+static int probe_info_stap(dtrace_hdl_t *dtp, const dt_probe_t *prp,
+			   int *argcp, dt_argdesc_t **argvp)
+{
+	int		i, j;
+	char		*p;
+	list_probe_t	*pup = prp->prv_data;
+	dt_uprobe_t	*upp;
+	size_t		argc = 0;
+	dt_argdesc_t	*argv = NULL;
+
+	/* No underlying probes?  No args.  */
+	if (!pup)
+		goto done;
+
+	upp = pup->probe->prv_data;
+	if (!upp || upp->sargv == NULL)
+		goto done;
+
+	/* First count the arguments. */
+	for (p = upp->sargv; p != NULL; argc++) {
+		p = strchr(p, '@');
+		if (p++ == NULL)
+			break;
+	}
+
+	/* Record number of arguments, and allocate descriptors. */
+	upp->sargc = argc;
+	if (argc == 0)
+		goto done;
+
+	argv = dt_calloc(dtp, argc, sizeof(dt_argdesc_t));
+	if (argv == NULL)
+		return dt_set_errno(dtp, EDT_NOMEM);
+
+	/* Fill in argument data. */
+	for (i = 0, p = upp->sargv; i < argc; i++) {
+		char	*q, *r, *type;
+
+		q = r = strchr(p, '@');
+		for (q--; q >= p && (*q == '-' || isdigit(*q)); q--) ;
+		if (q < p)
+			q = p;
+
+		type = staptype(atoi(q));
+		if (type == NULL)
+			goto oom;
+
+		argv[i].native = type;
+		argv[i].mapping = i;
+
+		p = r + 1;
+	}
+
+done:
+	*argcp = argc;
+	*argvp = argv;
+
+	return 0;
+oom:
+
+	for (j = 0; j <= i; j++)
+		free((char *) argv[i].native);
+
+	dt_free(dtp, argv);
+
+	return dt_set_errno(dtp, EDT_NOMEM);
+}
+
 /*
  * Destroy a uprobe for a given device and address.
  */
@@ -1834,6 +2007,12 @@ static void detach(dtrace_hdl_t *dtp, const dt_probe_t *uprp)
 	uprobe_delete(upp->dev, upp->inum, upp->off, upp->flags);
 }
 
+/* Clean up the private provider data. */
+static void destroy(dtrace_hdl_t *dtp, void *arg)
+{
+	dt_htab_destroy((dt_htab_t *)arg);
+}
+
 /*
  * Used for underlying probes (uprobes).
  */
@@ -1873,4 +2052,18 @@ dt_provimpl_t	dt_usdt = {
 	.probe_destroy	= &probe_destroy,
 	.discover	= &discover,
 	.add_probe	= &add_probe_usdt,
+	.destroy	= &destroy,
+};
+
+/*
+ * Used for stapsdt probes.
+ */
+dt_provimpl_t	dt_stapsdt = {
+	.name		= "stapsdt",
+	.prog_type	= BPF_PROG_TYPE_UNSPEC,
+	.provide_probe	= &provide_stapsdt_probe,
+	.enable		= &enable_stapsdt,
+	.probe_info	= &probe_info_stap,
+	.probe_destroy	= &probe_destroy,
+	.add_probe	= &add_probe_usdt,
 };
