@@ -1,7 +1,7 @@
                    FUNCTION:NAME
                           :BEGIN 
              0  1  2  3  4  5  6  7  8  9  a  b  c  d  e  f  0123456789abcdef
-         0: 34 12 78 56 00 00 65 87 21 43                    4.xV..e.!C
+         0: 64 74 72 61 63 65 34 12 21 43                    dtrace4.!C
 
 
 -- @@stderr --
